@@ -47,7 +47,7 @@ use SDL::Tutorial::3DWorld::OpenGL ();
 use SDL::Surface     ();
 use SDL::PixelFormat ();
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 =pod
 
@@ -108,8 +108,8 @@ sub file {
 sub init {
 	my $self = shift;
 
-	# Enable texture support
-	OpenGL::glEnable( OpenGL::GL_TEXTURE_2D );
+	# Shortcut if already initiated
+	return if defined $self->{id};
 
 	# Use SDL to load the image
 	my $image = SDL::Image::load( $self->file );
@@ -197,18 +197,6 @@ sub init {
 sub display {
 	# Rebind the texture using the previously allocated id
 	OpenGL::glBindTexture( OpenGL::GL_TEXTURE_2D, $_[0]->{id} );
-
-	# As there is a mix of textured and plain objects in the 3DWorld
-	# tutorial, there is no default setting for texturing.
-	# Each object will need to flip it on or off as appropriate.
-	OpenGL::glEnable( OpenGL::GL_TEXTURE_2D );
-
-	# Reset color to opaque white to ensure that the texture is drawn
-	# fully opaque and in full colour.
-	# Having a non-white colour when drawing a textured polygon will
-	# result in the texture colours being filtered through that colour.
-	# This can be cool sometimes, but it's not what we want by default.
-	OpenGL::glColor4f( 1, 1, 1, 1 );
 }
 
 1;
